@@ -43,7 +43,7 @@ class EnvironmentManager(base.ResourceManager):
         # read it's contents first.
         if 'file' in kwargs:
             file = kwargs['file']
-            kwargs = utils.load_content(utils.get_contents_if_file(file))
+            kwargs = utils.load_content(self.get_contents_if_file(file))
 
         self._ensure_not_empty(name=kwargs.get('name', None),
                                variables=kwargs.get('variables', None))
@@ -59,7 +59,7 @@ class EnvironmentManager(base.ResourceManager):
         # read it's contents first.
         if 'file' in kwargs:
             file = kwargs['file']
-            kwargs = utils.load_content(utils.get_contents_if_file(file))
+            kwargs = utils.load_content(self.get_contents_if_file(file))
 
         name = kwargs.get('name', None)
         self._ensure_not_empty(name=name)
